@@ -3,6 +3,18 @@ from csc_utils.seq import sampling_sequence
 
 #__all__ = ['chunk', 'Status', 'ForEach', 'foreach', 'queryset_foreach', 'cached', 'sampling_sequence']
 
+def in_groups_of(size, iterator):
+    counter = 0
+    acc = []
+    for item in iterator:
+        acc.append(item)
+        counter += 1
+        if counter == size:
+            yield acc
+            acc = []
+            counter = 0
+    if acc: yield acc
+
 def run_doctests(mod):
     """
     Run the doctests in a module. This is a great thing to call from a
